<% link_asset "application.js" %>
<% link_asset "application.css" %>
<% link_asset "POW.png" %>

define("application.js", <%= environment["application.js"].digest_path.inspect %>)
define("application.css", <%= environment["application.css"].digest_path.inspect %>)
define("POW.png", <%= environment["POW.png"].digest_path.inspect %>)
