<% depend_on(File.expand_path('../dependencies', __FILE__)) %>
<% depend_on(File.expand_path('../dependencies/a.js', __FILE__)) %>
<% depend_on(File.expand_path('../dependencies/b.js', __FILE__)) %>
<% depend_on(File.expand_path('../dependencies/c.js', __FILE__)) %>
