div {
   url: asset-url("foo.svg");
   url: image-url("foo.png");
   url: video-url("foo.mov");
   url: audio-url("foo.mp3");
   url: font-url("foo.woff2");
   url: font-url("foo.woff");
   url: javascript-url("foo.js");
   url: stylesheet-url("foo.css");
}