@font-face {
  font-family: 'octicons';
  src: font-url('octicons.eot?#iefix') format('embedded-opentype'),
       font-url('octicons.woff2') format('woff2'),
       font-url('octicons.woff') format('woff'),
       font-url('octicons.ttf') format('truetype'),
       font-url('octicons.svg#octicons') format('svg');
  font-weight: normal;
  font-style: normal;
}
