<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8">
    <link href="<%= link_asset('gallery.css').digest_path %>" rel="stylesheet">
  </head>
  <body>
    <h1>Hello, World!</h1>
    <script src="<%= link_asset('gallery.js').digest_path %>"></script>
  </body>
</html>
