/*
 *= depend_on "POW.png"
 */

<%
  path = File.expand_path("../POW.png", __FILE__)
  data = [File.open(path, "rb") { |f| f.read }].pack('m')
%>
.pow {
  background: url(data:image/png;base64,<%= data %>) no-repeat;
}
